<?php
require_once APP_PATH . '/View/ViewModel.php';

class AbstractController
{
    protected $view = null;
    protected $controllerName = '';
    protected $actionName     = '';
    private $arrLoadedModel   = array();
    
    
    /**
     * Init controller
     * 
     * @param string $controller
     * @param string $action
     */
    public function init($controller, $action)
    {
        $this->controllerName = $controller;
        $this->actionName     = $action;
        $this->view           = new ViewModel();
    }
    
    
    /**
     * Default action when controller init
     */
    public function indexAction()
    {
        
    }
    
    
    /**
     * Render view
     */
    public function render()
    {        
        echo $this->view->renderController($this->controllerName, $this->actionName);
    }
    
    
    protected function loadModel($name)
    {
        if (isset($this->arrLoadedModel[$name])) {
            return $this->arrLoadedModel[$name];
        }
        
        $path = APP_PATH . '/Model/' . $name . '.php';
        
        if (file_exists($path)) {
            include_once $path;
        }
        else {
            throw new Exception('Couldn\'t load model ' . $name);
        }
        
        if (! class_exists($name)) {
            throw new Exception('Class ' . $name . ' is not exist.');
        }
        
        $this->arrLoadedModel[$name] = new $name();
        
        return $this->arrLoadedModel[$name];
    }
    
    
    /**
     * Get param from url
     * 
     * @param string $name
     * @param string $default
     * @return mixed|NULL
     */
    protected function getParam($name, $default = null)
    {
        if (isset($_GET[$name])) {
            return $_GET[$name];
        }
        
        return null;
    }
    
    
    /**
     * Get post data
     * 
     * @param string $name
     * @param string $default
     * @return mixed|NULL
     */
    protected function getPost($name, $default = null)
    {
        if (isset($_POST[$name])) {
            return $_POST[$name];
        }
        
        return null;
    }
    
    
    /**
     * check is post method
     * 
     * @return boolean
     */
    protected function isPost()
    {
        return $_SERVER['REQUEST_METHOD'] === 'POST';
    }
}