<?php


abstract class ModelAbstract
{
    protected $conn  = null;
    protected $table = null;
    protected $lastInsertId = null;
    
    
    public function __construct()
    {
        $this->conn = DbConnection::getInstance();
    }
    
    
    public function insert(array $data)
    {
        // connect database
        $db     = $this->getDbAdapter();;
        
        // get key insert
        $arrKey = array_keys($data);
        
        // get quotes value
        $calQuote = function ($value) use ($db) {
            return $db->quote($value);
        };
        $arrVal = array_map($calQuote, $data);
        
        $sql = sprintf('INSERT INTO %s(%s) VALUES(%s)', $this->table, implode(',', $arrKey), implode(',', $arrVal));
        
        if ($db->query($sql)) {
            $this->lastInsertId = $db->lastInsertId();
            return true;
        }
        
        return false;
    }
    
    
    
    public function update(array $data, $condition = null)
    {
        $sql = sprintf('UPDATE %s SET %s WHERE %s', $this->table, $this->buildSql($data, ','), $this->buildSql($condition));

        return $this->getDbAdapter()->query($sql);
    }
    
    
    private function buildSql(array $where, $sep = ' AND ')
    {
        $sWhere     = '';
        $arrWhere   = array();
        $db         = $this->getDbAdapter();
        
        foreach ($where as $field => $val)
        {
            $arrWhere[] = sprintf('`%s` = %s', $field, $db->quote($val));
        }
        
        
        if (! empty($arrWhere)) {
            return implode($sep, $arrWhere);
        }
        
        return 1;
    }
    
    
    public function getDbAdapter()
    {
        return $this->conn->connect();
    }
}